/*
 * Copyright (c) 2008-present Sonatype, Inc.
 *
 * All rights reserved. Includes the third-party code listed at http://links.sonatype.com/products/nexus/pro/attributions
 * Sonatype and Sonatype Nexus are trademarks of Sonatype, Inc. Apache Maven is a trademark of the Apache Foundation.
 * M2Eclipse is a trademark of the Eclipse Foundation. All other trademarks are the property of their respective owners.
 */
/*global Ext, NX*/

/**
 * Datastores controller.
 *
 * @since 3.19
 */
Ext.define('NX.proui.controller.Datastores', {
  extend: 'NX.controller.Drilldown',
  requires: [
    'NX.Dialogs',
    'NX.Messages',
    'NX.Permissions',
    'NX.I18n'
  ],
  masters: [
    'nx-proui-datastore-list'
  ],
  models: [
    'DatastoreConfiguration'
  ],
  stores: [
    'DatastoreSource',
    'DatastoreType',
    'DatastoreConfiguration',
    'ModifiableDatastoreSource'
  ],
  views: [
    'datastore.DatastoreAdd',
    'datastore.DatastoreFeature',
    'datastore.DatastoreList',
    'datastore.DatastoreSettings',
    'datastore.DatastoreSettingsForm'
  ],
  refs: [
    { ref: 'feature', selector: 'nx-proui-datastore-feature' },
    { ref: 'content', selector: 'nx-feature-content' },
    { ref: 'list', selector: 'nx-proui-datastore-list' },
    { ref: 'settings', selector: 'nx-proui-datastore-feature nx-proui-datastore-settings' }
  ],
  icons: {
    'datastore-default': {
      file: 'database_share.png',
      variants: ['x16', 'x32']
    }
  },

  permission: 'nexus:datastores',

  /**
   * @override
   */
  init: function() {
    var me = this;

    me.features = {
      mode: 'admin',
      path: '/Repository/Datastores',
      text: NX.I18n.get('Datastores_Text'),
      description: NX.I18n.get('Datastores_Description'),
      view: {xtype: 'nx-proui-datastore-feature'},
      iconConfig: {
        file: 'database_share.png',
        variants: ['x16', 'x32']
      },
      visible: function() {
        return NX.Permissions.check('nexus:datastores:read') && !Ext.isEmpty(NX.State.getUser());
      }
    };

    me.callParent();

    me.listen({
      controller: {
        '#Refresh': {
          refresh: me.loadStores
        }
      },
      store: {
        '#DatastoreConfiguration': {
          load: me.reselect
        }
      },
      component: {
        'nx-proui-datastore-list': {
          beforerender: me.loadRecipe
        },
        'nx-proui-datastore-list button[action=new]': {
          click: me.showAddWindow
        },
        'nx-proui-datastore-settings button[action=save]': {
          click: me.updateDatastore
        },
        'nx-proui-datastore-settings-form': {
          submitted: me.loadStores
        }
      }
    });
  },

  /**
   * @override
   */
  getDescription: function(model) {
    return model.get('name');
  },

  /**
   * @override
   */
  onSelection: function(list, model) {
    if (Ext.isDefined(model)) {
      this.getSettings().loadRecord(model);
    }
  },

  /**
   * @protected
   * Enable 'Delete' when user has 'delete' permission for selected data store.
   */
  bindDeleteButton: function(button) {
    var me = this;

    button.mon(
        NX.Conditions.and(
            NX.Conditions.isPermitted(this.permission + ':delete'),
            NX.Conditions.watchEvents([
              { observable: me.getStore('DatastoreConfiguration'), events: ['load']},
              { observable: Ext.History, events: ['change']}
            ], me.watchEventsHandler())
        ),
        {
          satisfied: function () {
            button.enable();
          },
          unsatisfied: function () {
            button.disable();
          }
        }
    );
  },

  /**
   * @private
   */
  watchEventsHandler: function () {
    var me = this,
        store = me.getStore('DatastoreConfiguration');

    return function() {
      var datastoreId = me.getModelIdFromBookmark(),
          model = datastoreId ? store.findRecord('name', datastoreId, 0, false, true, true) : undefined;

      if (model) {
        var inUse = model.get('inUse');
        if (inUse) {
          me.showInfo(NX.I18n.format('Datastore_DatastoreFeature_Delete_Disabled_Message'));
          return false;
        }

        me.clearInfo();
        return true;
      }

      return false;
    };
  },

  /**
   * @private
   */
  showAddWindow: function() {
    var me = this;

    // Show the first panel in the create wizard, and set the breadcrumb
    me.setItemName(1, NX.I18n.get('Datastores_Create_Title'));
    me.loadCreateWizard(1, Ext.create('widget.nx-proui-datastore-add'));
  },

  /**
   * @private
   */
  loadRecipe: function() {
    var me = this,
        list = me.getList();

    if (list) {
      me.getStore('DatastoreConfiguration').clearFilter();
      me.getStore('DatastoreSource').load();
      me.getStore('DatastoreType').load();
    }
  },

  /**
   * @private
   * Updates data store.
   */
  updateDatastore: function(button) {
    var me = this,
        form = button.up('form'),
        values = form.getValues();

    NX.Dialogs.askConfirmation(NX.I18n.get('Datastore_DatastoreFeature_Update_Title'),
                               NX.I18n.get('Datastore_DatastoreFeature_Update_Warning'),
                               function () {
      me.getContent().getEl().mask(NX.I18n.get('Datastores_Update_Mask'));
      NX.direct.proui_Datastore.update(values, function(response) {
        me.getContent().getEl().unmask();
        if (Ext.isObject(response)) {
          if (response.success) {
            NX.Messages.success(NX.I18n.format('Datastores_Update_Success',
                  me.getDescription(me.getDatastoreConfigurationModel().create(response.data))));
            me.getStore('DatastoreConfiguration').load();
          }
          else if (Ext.isDefined(response.errors)) {
            form.markInvalid(response.errors);
          }
        }
      });
    }, {scope: me});
  },

  /**
   * @private
   */
  deleteModel: function(model) {
    var me = this,
        description = me.getDescription(model);

    me.getContent().getEl().mask(NX.I18n.get('Datastores_Delete_Mask'));
    NX.direct.proui_Datastore.remove(model.getId(), function(response) {
      me.getContent().getEl().unmask();
      me.getStore('DatastoreConfiguration').load();
      if (Ext.isObject(response) && response.success) {
        NX.Messages.success(NX.I18n.format('Datastores_Delete_Success', description));
      }
    });
  }
});
